/**
 * This Mongo Redis Topic Consumer uses a mongo collection to persistenly represent a message stream.
 * Each document in the collection represents a message within the topic.
 *
 * A redis connection is used to broadcast signals when new messages arrive to unlock waiting consumers.
 * This allows us to avoid a polling mechanism inside the consumer code.
 */
import { TopicConsumer } from "../../topicConsumer";
import { TopicMessageDocument, TopicMessageIdentifier } from "../../types";
export declare class MongoRedisTopicConsumer<T> extends TopicConsumer<T> {
    private mongoUrl;
    private databaseName;
    private collectionName;
    private redisHost;
    private redisPort;
    private client;
    private redis;
    private isStarting;
    private isStreaming;
    private isPolling;
    private isMoreMessages;
    private isStopped;
    private debuggingHandlers;
    private crashedHandlers;
    private hasCalledNoMoreMessages;
    private pollingLoop;
    constructor(mongoUrl: string, databaseName: string, collectionName: string, redisHost: string, redisPort: number);
    start(): Promise<void>;
    stop(): Promise<void>;
    addDebuggingHandler(handler: (type: string, message: string) => void): void;
    private fireDebug;
    private getCollection;
    streamMessagesFrom(messageCallback: (message: TopicMessageDocument<T>) => Promise<void>, lastId: TopicMessageIdentifier | null | undefined, noMoreMessagesCallback: () => void, crashCallback: (e: Error) => void): void;
    private pollForNewMessages;
    private fetchNextMessagesFromMongo;
    private startListeningForNewMessageSignal;
}
